@@ -32,8 +32,7 @@ __all__ = ['EventPayload',
            'dataobject',
            'DataObject',
            'copy_payload',
-           'jsonify',
-           'validation']
+           'jsonify']
 
 
 @dataclass
@@ -82,6 +81,27 @@ EventPayload = Union[str, int, float, bool, dict, set, list, DataObject]
 EventPayloadType = TypeVar("EventPayloadType")
 
 
+class BinaryAttachment:
+    """Type descriptor for multipart binary attachments"""
+
+
+class BinaryDownload:
+    """
+    Type descriptor for binary download. Events returning files to download
+    should specialize this class for specififc content types, i.e.:
+
+    ```
+    @dataobject
+    @dataclass
+    class ImagePng(BinaryDownload):
+        content_type = "image/png"
+    ```
+
+    This way, the  type can be used in event API speficacation as reponse type.
+    """
+    content_type: str = "application/octet-stream"
+
+
 def dataobject(
         decorated_class=None, *,
         event_id: Optional[str] = None,
